/*
Copyright 2004 John Tsiombikas <nuclear@siggraph.org>

This file is part of the MUD project by The Lab Software.

MUD is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MUD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MUD; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <string.h>
#include "area.h"

/**
 * creates an area with the specified area id and returns the pointer
 */
struct area *create_area(area_id aid) {
	struct area *area;

	if(!(area = malloc(sizeof(struct area)))) {
		return 0;
	}

	area->aid = aid;
	area->title = area->desc = 0;

	return area;
}

/**
 * properly destroys an area
 */
void destroy_area(struct area *area) {
	free(area->title);
	free(area->desc);
	free(area);
}

/**
 * set the area title (the string is copied, so you don't need to keep it around)
 */
int set_area_title(struct area *area, const char *title) {
	char *temp;
	
	if(!(temp = malloc(strlen(title)+1))) {
		return -1;
	}

	free(area->title);
	area->title = temp;
	return 0;
}

/**
 * set the area description (the string is copied, so you don't need to keep it around)
 */
int set_area_description(struct area *area, const char *desc) {
	char *temp;
	
	if(!(temp = malloc(strlen(desc)+1))) {
		return -1;
	}

	free(area->desc);
	area->desc = temp;
	return 0;
}
