/*
Copyright 2004 John Tsiombikas <nuclear@siggraph.org>

This file is part of the MUD project by The Lab Software.

MUD is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MUD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MUD; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file item.h */

#ifndef _ITEM_H_
#define _ITEM_H_

#include "item_types.h"
#include "combat.h"

/* this is ugly, but helps implement polymorphic behaviour easily */
#define ITEM_MEMBER_VARS \
	char *name, *desc;\
	enum item_type type;\
	unsigned int weight

/** genric item struct, abstract base class we could say */
struct item {
	ITEM_MEMBER_VARS;
};

/** food with some nutritional value and optional magical properties */
struct food {
	ITEM_MEMBER_VARS;
	int value;
};

/** drink with some anti-thirst value and optional magical properties */
struct drink {
	ITEM_MEMBER_VARS;
	int value;
};

/** gear type, anything that we can wear */
struct gear {
	ITEM_MEMBER_VARS;
	enum gear_type gear_type;

	int ac;
};

/** weapons */
struct weapon {
	ITEM_MEMBER_VARS;
	enum damage_type damage_type;
	
	int range;	/* 0 to signify non-ranged weapons */
	int ammo;	/* for ranged weapons */

	enum dice damage_dice;
	int dice_count;
	int magic_bonus;
};

/** scrolls/books/etc */
struct text {
	ITEM_MEMBER_VARS;
	char *text;	
};

/** items that contain other items */
struct container {
	ITEM_MEMBER_VARS;
	int count, max;
	struct item **items;
};

void set_item_name(struct item *item, const char *name);
void set_item_desc(struct item *item, const char *desc);
void set_text_contents(struct text *text, const char *contents);

void destroy_item(struct item *item);
struct container *create_container(int size);
void destroy_container(struct container *cont);

#endif	/* _ITEM_H_ */
