/*
Copyright 2004 John Tsiombikas <nuclear@siggraph.org>

This file is part of the MUD project by The Lab Software.

MUD is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MUD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MUD; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file item_types.h */

#ifndef _ITEM_TYPES_H_
#define _ITEM_TYPES_H_

/** item categories */
enum item_type {
	ITEM_UNK,		/**< unknown item */
	ITEM_FOOD,		/**< food or other edible */
	ITEM_DRINK,		/**< drink/drinkable substance */
	ITEM_GEAR,		/**< anything wearable */
	ITEM_WEAPON,	/**< anything that can cause damage except magic */
	ITEM_TEXT,		/**< anything that you can read, scrolls/books/maps */
	ITEM_CONTAINER	/**< meta-item :) */
};

/* take care to change this if the above list is modified */
#define ITEM_TYPE_COUNT		7

/** gear types (where you can wear it) */
enum gear_type {
	GEAR_HEAD,		/**< hats, helmets, etc */
	GEAR_ARM,		/**< arm protection */
	GEAR_HAND,		/**< gloves, rings, gauntlets, etc */
	GEAR_TORSO,		/**< breastplates, chain shirts etc */
	GEAR_WAIST,		/**< belts mainly */
	GEAR_LEG,		/**< leg stuff */
	GEAR_FOOT,		/**< shoes, boots, etc */
	GEAR_BACK		/**< backpacks and such */
};

/* take care to change this if the above list is modified */
#define GEAR_TYPE_COUNT		8

enum damage_type {
	DAMAGE_SLASH,
	DAMAGE_PIERCE,
	DAMAGE_BLUNT
};

/* definition in item.c */
extern unsigned int item_max_use_count[ITEM_TYPE_COUNT];
extern unsigned int gear_max_use_count[GEAR_TYPE_COUNT];

#endif	/* _ITEM_TYPES_H_ */
