#include <stdlib.h>
#include "list.h"

/** 
 * delete the whole list, calling a custom deallocation function
 * on the generic data pointers, if one is provided.
 */
void delete_list(struct list_node *list, void (*delete_data)(void*)) {
	while(list) {
		struct list_node *tmp = list;
		list = list->next;
		if(delete_data) delete_data(tmp->data);
		free(tmp);
	}
}

/** 
 * adds a new node to the list, containing the data pointer specified
 * at the second argument
 */
void add_list_node(struct list_node **list, void *data) {
	struct list_node *node = malloc(sizeof(struct list_node));

	node->data = data;
	node->next = *list;
	node->prev = 0;
	if(*list) (*list)->prev = node;
	*list = node;
}

/**
 * removes the specified node from the list, optionally calling a custom
 * deallocation function provided at the second argument, on the data
 * of that node.
 * 
 * Returns the data pointer if no deallocation function is specified, NULL otherwise
 */
void *remove_list_node(struct list_node *node, void (*delete_data)(void*)) {
	void *data;

	node->next->prev = node->prev;
	node->prev->next = node->next;

	if(delete_data) {
		delete_data(node->data);
		node->data = 0;
	}
	data = node->data;
	free(node);

	return data;
}

/**
 * finds the specified data item on the list, the provided comparison function
 * is used to compare each element of the list with the second argument.
 * 
 * Returns the node if it finds a match, NULL otherwise
 */
struct list_node *find_list_node(struct list_node *list, void *data, int (*cmp_func)(void*, void*)) {

	while(list && cmp_func(data, list->data)) {
		list = list->next;
	}

	return list;
}

/**
 * counts and returns the number of elements in the list.
 */
unsigned int count_list_nodes(struct list_node *list) {
	unsigned int counter = 0;
	while(list) {
		counter++;
		list = list->next;
	}

	return counter;
}
