#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "world.h"
#include "err_msg.h"

#define INIT_GRAPH_TABLE_SIZE	64

static struct list_node **area_graph;
static struct area **area_table;
static int area_count;
static int graph_size = INIT_GRAPH_TABLE_SIZE;

/**
 * allocates the world data structures.
 * returns 0 on success, -1 on failiure
 */
int create_world(void) {
	if(!(area_graph = malloc(INIT_GRAPH_TABLE_SIZE * sizeof(struct list_node*))) ||
		!(area_table = malloc(INIT_GRAPH_TABLE_SIZE * sizeof(struct area*)))) {
		error("failiure at %s: %s", __func__, strerror(errno));
		return -1;
	}
	return 0;
}

/**
 * deallocates the world data structures
 */
void destroy_world(void) {
	int i;
	for(i=0; i<area_count; i++) {
		delete_list(area_graph[i], free);
		free(area_table[i]);
	}

	free(area_graph);
	free(area_table);
}

/**
 * get a list with the adjacent areas of the specified area
 */
struct list_node *get_adj_area_list(area_id aid) {
	return area_graph[aid];
}

/**
 * adds an already created area to the world data structures
 */
area_id add_area(struct area *area) {
	area_id aid;
	
	/* allocate more memory for the graph and the area table if needed */
	if(area_count == graph_size) {
		struct list_node **tmp;
		struct area **tmp_area;
		
		if(!(tmp = realloc(area_graph, 2 * graph_size * sizeof(struct list_node*)))) {
			error("failiure at %s: %s", __func__, strerror(errno));
			return -1;
		}

		if(!(tmp_area = realloc(area_table, 2 * graph_size * sizeof(struct area*)))) {
			error("failiure at %s: %s", __func__, strerror(errno));
			return -1;
		}

		graph_size *= 2;
		area_graph = tmp;
		area_table = tmp_area;
	}

	/* and add the actual entry */
	aid = area_count++;
	area_graph[aid] = 0;
	area_table[aid] = area;
	area->aid = aid;

	return aid;
}

/**
 * creates a new area and then adds it to the world
 */
area_id add_new_area(void) {
	struct area *area;

	if(!(area = create_area(-1))) {
		return -1;
	}

	return add_area(area);
}

/**
 * get a pointer to the area specified by the area id
 */
struct area *get_area(area_id aid) {
	return area_table[aid];
}

/**
 * add a link from an area to another area in the graph
 */
int add_area_link(area_id aid, area_id adj) {
	add_list_node(&area_graph[aid], malloc(sizeof(int)));
	return 0;
}

static int cmp_ints(void *a, void *b) {
	int n1 = *(int*)a;
	int n2 = *(int*)b;
	return n1 == n2 ? 0 : (n1 < n2 ? -1 : 1);
}

/**
 * delete a link from an area to another area in the graph
 */
int del_area_link(area_id aid, area_id adj) {
	struct list_node *node = find_list_node(area_graph[aid], &adj, cmp_ints);

	if(!node) return -1;
	remove_list_node(node, free);
	return 0;
}
